/*************************************************************
*             program name: DN64.C                           *
*  use parallel EPP port to debug N64                        *
*  r/w DS1, r/w eeprom, read game cart                       *
*                                                            *
* ai[]=0 r/w a[7..0]                                         *
* ai[]=1 r/w a[15..8]                                        *
* ai[]=2 r/w a[23..16]                                       *
* ai[]=3 r/w a[31..24]                                       *
* ai[]=4 r/w data                                            *
* ai[]=5 w/o d4=!rst output pin                              *
*        w/o d3=en 1=output ras/cas address to ad[], 0=none  *
*        w/o d2=rc 1=ras address, 0=cas address              *
*        w/o d1=cas output pin                               *
*        w/o d0=ras output pin                               *
* ai[]=6 r/w d0=sd 1=hi-z, 0=low                             *
*        w/o d1=sck output pin                               *
*************************************************************/
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <dir.h>

#define ai 0x37b
#define data 0x37c
#define trans_size 32768
#define set_ai_write outportb(0x37a,5);		// ninit=1, nwrite=0
#define set_data_write outportb(0x37a,1);	// ninit=0, nwrite=0
#define set_data_read outportb(0x37a,0);	// ninit=0, nwrite=1
#define set_normal outportb(0x37a,4);		// ninit=1, nwrite=1

// for read/write eeprom
#define eeprom_size 512


char file_name[128],wv_mode;
unsigned char eeprom_sck_sd;		// d1=>sck, d0=sd
unsigned int eeprom_index;
FILE *fptr;
union mix_buffer {
unsigned char buffer[32768];
unsigned int bufferx[16384];
}mix;
unsigned int hi_word,i,j;

/**************************************
*               Subroutine            *
**************************************/
void disp_buffer(unsigned int disp_len)
{
   int i,j,x,y;
   for (i=0; i<disp_len ; i++)
      {
      if ((i & 0xf)==0)
	 printf("%04x : ",i&0xfff0);
      if ((i & 0xf)==8)
	 printf("- ");
      printf("%02x ",mix.buffer[i]);
      if ((i & 0xf)==0xf)
	 {
	 printf("-> ");
	 for (j=0; j<16 ; j++)
	    {
	    if (mix.buffer[(i&0xfff0)+j]<0x20 || mix.buffer[(i&0xfff0)+j]>0x80)
	       printf (".");
	    else
	       printf ("%c",mix.buffer[(i&0xfff0)+j]);
	    }
	 printf("\n");
	 }
      }

   y=disp_len&0xf;
   if (y)				/* not equal 16*?? */
      {
      for (x=y;x<16;x++)
	 {
	 if ((x & 0xf)==8)
	    printf("- ");
	 printf("   ");
	 if ((x & 0xf)==0xf)
	    {
	    printf("-> ");
	    for (j=0; j<y ; j++)
	       {
	       if (mix.buffer[(i&0xfff0)+j]<0x20 || mix.buffer[(i&0xfff0)+j]>0x80)
		  printf (".");
	       else
		  printf ("%c",mix.buffer[(i&0xfff0)+j]);
	       }
	    printf("\n");
	    }
	 }
      }
}
void set_ai(unsigned char _ai)
{
   set_ai_write			// ninit=1, nwrite=0
   outportb(ai,_ai);
}

void set_ai_data(unsigned char _ai,unsigned char _data)
{
   set_ai(_ai);
   set_data_write		// ninit=0, nwrite=0
   outportb(data,_data);
}

void send_ras_cas(void)
{
   set_ai_data(5,0x0f);			// en=1,cr=1,cas=1,ras=1
   set_ai_data(5,0x0e);			// en=1,cr=1,cas=1,ras=0
   set_ai_data(5,0x0a);			// en=1,cr=0,cas=1,ras=0
   set_ai_data(5,0x08);			// en=1,cr=0,cas=0,ras=0
   set_ai_data(5,0x00);			// en=0,cr=0,cas=0,ras=0
}


void init_port(void)
{
   outportb(0x379,1);		// clear EPP time flag
   set_ai_data(5,0x00);		// en,cr,cas,ras all=0 rst=hi
   eeprom_sck_sd=3;		// sck=1, sd=hi-z
   set_ai_data(6,3);
}

void end_port(void)
{
   set_ai_data(5,3);		// en=0,cr=0,cas=1,ras=1, rst=hi
   eeprom_sck_sd=3;		// sck=1, sd=hi-z
   set_ai_data(6,3);
   set_normal			// ninit=1, nWrite=1
}


//********************************************************************
// for read/write eeprom sub-routine
//********************************************************************
void eeprom_ck_lo(void)
{
   eeprom_sck_sd = eeprom_sck_sd & 1;	// sck=0, sd=no change
   set_ai_data(6,eeprom_sck_sd);
}
void eeprom_ck_hi(void)
{
   eeprom_sck_sd = eeprom_sck_sd | 2;	// sck=1, sd=no change
   set_ai_data(6,eeprom_sck_sd);
}
void eeprom_sd_lo(void)
{
   eeprom_sck_sd = eeprom_sck_sd & 2;	// sck=no change, sd=0
   set_ai_data(6,eeprom_sck_sd);
}
void eeprom_sd_hi(void)
{
   eeprom_sck_sd = eeprom_sck_sd | 1;	// sck=no change, sd=1
   set_ai_data(6,eeprom_sck_sd);
}
unsigned char eeprom_get_sd(void)
{
   set_ai(6);
   set_data_read			// ninit=0, nwrite=1
   return(inportb(data) & 1);			// read sd in d0
}

void gen_eeprom_read_gap(void)
{
   int i;
   for (i=0;i<100;i++) {	// the Gap at least 70 cycle
      eeprom_ck_lo();		// sck=0, sd=no change
      eeprom_ck_hi();		// sck=1, sd=no change
   }
}

void gen_eeprom_write_gap(void)
{
   int i;
   for (i=0;i<2000;i++) {	// the Gap at least 2000 cycle
      eeprom_ck_lo();		// sck=0, sd=no change
      eeprom_ck_hi();		// sck=1, sd=no change
   }
}

void gen_eeprom_clock(void)
{
   long int i;
   for (i=0;i<100000;i++) {
      eeprom_ck_lo();		// sck=0, sd=no change
      eeprom_ck_hi();		// sck=1, sd=no change
   }
}

void eeprom_init(void)
{
   set_ai_data(5,0x10);		// rst=0
   gen_eeprom_read_gap();
   set_ai_data(5,0);		// rst=1
   gen_eeprom_read_gap();
}

void write_eeprom_modulate(unsigned char datax)
{
   int i;
   for (i=0;i<8;i++) {
      eeprom_ck_lo();		// sck=0, sd=no change
      if (datax & 0x80) {
	 eeprom_sd_hi();        // sck=no change, sd=1
      }
      else {
	 eeprom_sd_lo();	// sck=no change, sd=0
      }
      eeprom_ck_hi();		// sck=1, sd=no change
      datax = datax << 1;
   }
}

void write_eeprom_bit(unsigned char bitx)
{
    if (bitx)
       write_eeprom_modulate(0x3f);	// modulate 1
    else
       write_eeprom_modulate(0x03);	// modulate 0
}

void write_eeprom_byte(unsigned char bytex)
{
   int i;
   for (i=0;i<8;i++) {
      if (bytex & 0x80)
	 write_eeprom_bit(1);
      else
	 write_eeprom_bit(0);
      bytex=bytex << 1;
   }
}

void read_eeprom_data(int mode)		// mode:0=dummy read,1=read,2=verify
{
   int i,j,k,loop;
   unsigned char datax,bytex;
   eeprom_sd_hi();		// sck=no change, sd=hi-z for read
   i=0;
   while(i<100) {
      eeprom_ck_lo();		// sck=0, sd=no change
      eeprom_ck_hi();		// sck=1, sd=no change
//      if (inportb(0x37a)&1) //******************
	if (!eeprom_get_sd())   // get sd status=0
	  break;
      i++;
   }
//   printf("i=%d sd=%d\n",i,eeprom_get_sd());
   if (i<100) {			// some data in bus
      loop=1;
      while(loop) {
	 bytex=0;
	 for (k=0;k<8;k++) {
	    datax=0;			// clear data to 0
	    for (j=0;j<8;j++) {
	       datax=datax<<1;
//	       if (!(inportb(0x37a)&1)) //***************
	       if (eeprom_get_sd())   // get sd status=1
		  datax=datax|1;
	       eeprom_ck_lo();		// sck=0, sd=no change
	       eeprom_ck_hi();		// sck=1, sd=no change
	    }
//	 printf("%02x ",data);
	    if ((datax==0x3f)||(datax==0x03)) {
	       bytex=bytex<<1;
	       if (datax==0x3f)
		  bytex=bytex|1;
	    }
	    else
	       loop=0;
	 }
	 if (loop) {
	    printf("%02x ",bytex);
	    if (mode)
	       if (eeprom_index<eeprom_size) {		// rom size=512
		  if (mode==1) {		// read
		     mix.buffer[eeprom_index]=bytex;
		     eeprom_index++;
		  }
		  else {
		     if (mix.buffer[eeprom_index]==bytex)	// verify
			eeprom_index++;
		     else {
			printf("%04x : %02x(org) %02x(rom)\07\n",eeprom_index,mix.buffer[eeprom_index],bytex);
			break;
		     }
		  }
	       }
	       else {
		  printf("over rom_size");
		  loop=0;
		  break;
	       }
	 }
	 else
	    printf("\n");
      }
   }
   else
      printf("no data read\n");
   gen_eeprom_read_gap();
}

void chk_eeprom_ok(void)
{
   printf("00 ===>");
   gen_eeprom_read_gap();
   write_eeprom_byte(0x00);
   write_eeprom_bit(0x01);
   read_eeprom_data(0);		// don't store data to disp_buffer
}

void read_eeprom_all(void)
{
   int i;
   eeprom_index=0;			// reset display buffer pointer to 0
   eeprom_init();			// rst from 0 to 1
   chk_eeprom_ok();
   for (i=0;i<(eeprom_size/8);i++)
      {
      printf("04,%02x ===>",i);
      write_eeprom_byte(0x04);
      write_eeprom_byte(i);
      write_eeprom_bit(0x01);
      read_eeprom_data(1);		// 0=dummy read,1=read,2=verify
      }
   chk_eeprom_ok();
   gen_eeprom_clock();
   printf("\n");
}

void verify_eeprom_all(void)
{
   int i;
   eeprom_index=0;			// reset display buffer pointer to 0
   eeprom_init();			// rst from 0 to 1
   chk_eeprom_ok();
   for (i=0;i<(eeprom_size/8);i++)
      {
      printf("04,%02x ===>",i);
      write_eeprom_byte(0x04);
      write_eeprom_byte(i);
      write_eeprom_bit(0x01);
      read_eeprom_data(2);		// 0=dummy read,1=read,2=verify
      }
   chk_eeprom_ok();
   gen_eeprom_clock();
   printf("\n");
}

void write_eeprom_all(void)
{
   int i,j;
   eeprom_index=0;			// reset display buffer pointer to 0
   eeprom_init();			// rst from 0 to 1
   chk_eeprom_ok();
   for (i=0;i<(eeprom_size/8);i++)
      {
      printf("05,%02x ===>",i);
      write_eeprom_byte(0x05);
      write_eeprom_byte(i);
      for (j=0;j<8;j++)
	 {
	 write_eeprom_byte(mix.buffer[eeprom_index]);
	 eeprom_index++;
	 }
      write_eeprom_bit(0x01);
      read_eeprom_data(0);		// don't store data to disp_buffer
      gen_eeprom_write_gap();
      }
   chk_eeprom_ok();
   gen_eeprom_clock();
   printf("\n");
}
//********************************************************************
// end of read/write eeprom sub-routine
//********************************************************************



char write_32k(unsigned int hi_word, unsigned int lo_word)
{
   unsigned char unpass,pass1;
   unsigned int i,j;
   unsigned int fix,temp;
   init_port();
   set_ai_data(3,0x10|(hi_word>>8));
   set_ai_data(2,(hi_word & 0xff));
   for (i=0;i<0x40;i++){
	 set_ai_data(1,((i<<1)|lo_word));
	 set_ai_data(0,0);
	 set_ai(4);		// set address index=4
	 set_data_write		// ninit=0, nWrite=0
	 fix=i<<8;
	 for (j=0;j<256;j++){
	    outport(data,mix.bufferx[j+fix]);
	 }
   }
   end_port();
   return(0);
}

char read_32k(unsigned int hi_word, unsigned int lo_word)
{
   char unpass;
   unsigned int i,j;
   unsigned int fix;
   init_port();
   set_ai_data(3,0x10|(hi_word>>8));
   set_ai_data(2,(hi_word & 0xff));
   for (i=0;i<0x40;i++){
	 set_ai_data(1,((i<<1)|lo_word));
	 set_ai_data(0,0);
	 send_ras_cas();		// send ras/cas address
	 set_ai(4);
	 set_data_read			// ninit=0, nwrite=1
	 fix=i<<8;
	 for (j=0;j<256;j++){
	    mix.bufferx[j+fix]=inport(data);
	 }
   }
   end_port();
   return(0);
}

char verify_32k(unsigned int hi_word, unsigned int lo_word)
{
   char unpass;
   unsigned int i,j,temp;
   unsigned int fix;
   init_port();
   set_ai_data(3,0x10|(hi_word>>8));
   set_ai_data(2,(hi_word & 0xff));
   for (i=0;i<0x40;i++){
	 set_ai_data(1,((i<<1)|lo_word));
	 set_ai_data(0,0);
	 send_ras_cas();	// send ras/cas address
	 set_ai(4);
	 set_data_read			// ninit=0, nwrite=1
	 fix=i<<8;
	 for (j=0;j<256;j++){
	    temp=inport(data);
	    if (temp!=mix.bufferx[j+fix]){
//	       printf("verify error!!!\07\n");
//	       printf("%2x%2x dram=%x, buffer=%x\n",i,j*2,temp,mix.bufferx[j+fix]);
	       outportb(0x37a,0x0b);	// all pin=0 for debug
	       printf("#");
	       init_port();
	       return(1);
	    }
	 }
   }
   end_port();
   return(0);
}

void read_adr(void)
{
   set_ai_data(6,0x0a);		// enable pc mode
   set_ai_data(7,0x05);		// enable pc mode
   printf("\na[31..0]=");
   set_ai(3);
   set_data_read		// ninit=0, nwrite=1
   printf("%02x",inportb(data));
   set_ai(2);
   set_data_read		// ninit=0, nwrite=1
   printf("%02x",inportb(data));
   set_ai(1);
   set_data_read		// ninit=0, nwrite=1
   printf("%02x",inportb(data));
   set_ai(0);
   set_data_read		// ninit=0, nwrite=1
   printf("%02x\n",inportb(data));
   end_port();
}


void read_some(void)
{
   init_port();
   set_ai_data(0,0);
   set_ai_data(1,0);
   set_ai_data(2,0);
   set_ai_data(3,0x10);
   send_ras_cas();		// send row-adr, cor-adr
   set_ai(4);
   set_data_read		// ninit=0, nWrite=1
   for (i=0;i<64;i++) {
      mix.bufferx[i]=inport(data);
   }
   disp_buffer(64);
   read_adr();
}

unsigned char check_card(void)		// *****
{
   init_port();
   set_ai_data(3,0x12);
   set_ai_data(2,0x34);
   set_ai_data(1,0x56);
   set_ai_data(0,0x78);
   set_ai(3);
   set_data_read		// ninit=0, nwrite=1
   if (inportb(data)!=0x12) return(1);
   set_ai(2);
   set_data_read		// ninit=0, nwrite=1
   if (inportb(data)!=0x34) return(1);
   set_ai(1);
   set_data_read		// ninit=0, nwrite=1
   if (inportb(data)!=0x56) return(1);
   set_ai(0);
   set_data_read		// ninit=0, nwrite=1
   if (inportb(data)!=0x78) return(1);
   end_port();
   return(0);
}
unsigned char write_32k_file(void)
{
   if (fwrite((char *)mix.buffer,sizeof(char),trans_size,fptr)!=trans_size)
      {
      fclose(fptr);	/* write data error */
      return(1);
      }
   printf(".");
   return(0);
}

unsigned char read_32k_file()
{
   if (fread((char *)mix.buffer,sizeof(char),trans_size,fptr)!=trans_size)
      {
      printf("read error!!!\07\n");
      fclose(fptr);	/* read data error */
      return(1);
      }
   printf(".");
   return(0);
}

unsigned char load_32k_file(void)
{
   unsigned int j;
   printf("Input filename : ");
   scanf("%s",file_name);
   if((fptr=fopen(file_name,"r+b"))==NULL){/* open error */
      printf("open error !!!\07\n");
      return(1);
   }
   if (read_32k_file()!=0){
      printf("load file error\n");
      fclose(fptr);
      return(1);
   }
   printf("load file ok\n");
   fclose(fptr);
   return(0);
}

save_32k_file(void)
{
   printf("Input filename : ");
   scanf("%s",file_name);
   if((fptr=fopen(file_name,"w+b"))==NULL){		/* open error */
      printf("open error !!!\07\n");
      return(-1);
   }

   if (write_32k_file()!=0){
      printf("save file error\n");
      fclose(fptr);
      return(1);
   }
   printf("save file ok\n");
   fclose(fptr);
   return(0);
}

void set_adr(unsigned int ras, unsigned cas)	// *****
{
unsigned char temp;
   init_port();
   set_ai_data(0,(cas & 0xff));		// a[7..0]
   set_ai_data(1,((cas>>8) & 0xff));	// a[15..8]
   set_ai_data(2,(ras & 0xff));		// a[23..16]
   set_ai_data(3,((ras>>8) & 0xff));    // a[31..24]
   send_ras_cas();		// send row-adr, cor-adr
   set_ai(4);
   set_data_read		// ninit=0, nWrite=1
}

unsigned char read_verify_sram(unsigned char mode,unsigned int length)
{
   set_adr(hi_word,0x0000);		// default read mode

//   printf("hi_word=%x ",hi_word);
//   printf("read_data= ");
//      printf("%x ",read_data());
   if (mode){
      for (i=0;i<(length/2);i++){
	 mix.bufferx[i]=inport(data);		// read data to bufferx
//	 mix.bufferx[i]=read_data();		// read data to bufferx
      }
   }
   else{
      for (i=0;i<(length/2);i++){
//	 if (mix.bufferx[i]!=read_data()){	// verify data with bufferx
	 if (mix.bufferx[i]!=inport(data)){	// verify data with bufferx
//	    no_select();			// don't select any device
	    end_port();
	    printf("verify error at %x\n",i*2);
//	    disp_buffer(i*2);
	    return(1);
	 }
      }
   }
//   printf("\n");
//   no_select();			// don't select any device
     end_port();
//   disp_buffer(0x8000);
   return(0);
}

void write_sram(void)		// ******
{
   unsigned int k;
   set_adr(hi_word,0);
   set_data_write		// ninit=0, nWrite=0
   for (k=0;k<0x4000;k++){    		// 0x4000 word=0x8000 byte
      outport(data,mix.bufferx[k]);	// write data from bufferx
   }
   end_port();				// don't select any device
}

unsigned char verify_rom_32k(unsigned int hi_adr,unsigned int low_adr)
{
// hi_adr is high word(ras)
// low_adr is low word(cas)
   unsigned int no_bank,page_cnt,temp,fix;
   for (no_bank=0;no_bank<0x8000;no_bank+=0x200){// 64 pages x 512=32k
      set_adr(hi_adr,(no_bank|low_adr)); // device id=8 first rom bank
      fix=no_bank/2;
      for (page_cnt=0;page_cnt<256;page_cnt++){ 		// word operate
	 temp=inport(data);
//	     printf("adr=%x ",fix+page_cnt);
//	     printf("buffer=%x ",mix.bufferx[(fix+page_cnt)]);
//	     printf("port_data=%x\n",temp);
	 if (mix.bufferx[(fix+page_cnt)]!=temp){ // compare data with bufferx
//	 if (mix.bufferx[((no_bank>>1)|page_cnt)]!=inport(data)){ // compare data with bufferx
//	     printf("read rom error at %x\n",(fix+page_cnt));
//	     printf("buffer=%x ",mix.bufferx[(fix+page_cnt)]);
//	     printf("port_data=%x\n",temp);
//	     printf("^");
	     end_port();
	     return(1);				// verify error
	 }
      }
   }
   end_port();
   return(0);
}

unsigned char read_verify_rom_32k(unsigned int hi_adr,unsigned int low_adr,unsigned char mode)
{
// hi_adr is high word(ras)
// low_adr is low word(cas)
// mode 0=read only, 1=read & verify

   unsigned int no_bank,page_cnt,temp,fix;
//   printf("read rom\n");
   for (no_bank=0;no_bank<0x8000;no_bank+=0x200){// 64 pages x 512=32k
      set_adr(hi_adr,(no_bank|low_adr)); 	 // device id=8 first rom bank
      fix=no_bank/2;
      for (page_cnt=0;page_cnt<256;page_cnt++){  // word operate
	 mix.bufferx[(fix+page_cnt)]=inport(data);		// read data to bufferx
      }
   }

//   no_select();			// don't select any device
   end_port();
   if (mode==0) return(0);
   return(verify_rom_32k(hi_adr,low_adr));
}



unsigned char chk_rom_size(void)
{
   if (read_verify_rom_32k(0x1000,0,1)!=0)     // read the 1M first 32k
      return(0);		        // no n64 cartage
   if (mix.bufferx[0]!=0x8037)		// swapbyte 8037 was 3780
      return(0);			// no n64 cartage
   if (mix.bufferx[1]!=0x1240)		// swapbyte 1240 was 4012
      return(0);			// no n64 cartage
   if (read_verify_rom_32k(0x1080,0,1)!=0)	// read the 65M first 32k
      return(1);		        // 64M
   if (read_verify_rom_32k(0x10c0,0,1)!=0)	// read the 97M first 32k
      return(2);		        // 96M
   if (read_verify_rom_32k(0x1100,0,1)!=0)	// read the 129M first 32k
      return(3);		        // 128M
   if (read_verify_rom_32k(0x1180,0,1)!=0)	// read the 193M first 32k
      return(4);		        // 192M
   return(5);				// 256M
}

int write_file_32k(void)
{
   if (fwrite((char *)mix.buffer,sizeof(char),trans_size,fptr)!=trans_size)
      {
      fclose(fptr);	/* write data error */
      return(-1);
      }
   printf(".");
   return(0);
}

int backup_rom_all(unsigned int no_page)	//no_page=how many 32K
{
   unsigned int bank_32k,fix_code;
   if (no_page>0x400){
      printf("number of page great than 1024x32k(256M)\n");
      return(-1);
   }
   if((fptr=fopen(file_name,"w+b"))==NULL)
      {/* open error */
      printf("open error !!!\07\n");
      return(-1);
      }
   fix_code=0x1000;				// first 64M of rom
   for (bank_32k=0;bank_32k<no_page;bank_32k++){
      if ((bank_32k & 1))			// upper 32k
	 if (read_verify_rom_32k((fix_code|((bank_32k>>1)&0x1ff)),0x8000,1)){
	 printf("\tbackup error\07\n");
	 fclose(fptr);
	 return(-1);
	 }
      else
	 if (read_verify_rom_32k((fix_code|((bank_32k>>1)&0x1ff)),0x0000,1)){
	 printf("\tbackup error\07\n");
	 fclose(fptr);
	 return(-1);
	 }
      if (write_file_32k()!=0){
	 fclose(fptr);
	 return(-1);
      }
   }
   printf("\n");
   fclose(fptr);
   return(0);
}

void backup(void)
{
   unsigned char ch;		// single key function
   printf("checking rom size");
   ch=chk_rom_size();
   if (ch==0){
      printf(" -> no rom cartage present!!!\07\n");
      return;
   }
   printf(" -> rom size is ");
   switch (ch) {
      case 1: printf("64M");break;
      case 2: printf("96M");break;
      case 3: printf("128M");break;
      case 4: printf("192M");break;
      case 5: printf("256M");break;
   }
   printf("\n");
   read_verify_rom_32k(0x1000,0,0);		/* read first 32k */
   printf("card name is : ");
   for (ch=0x20;ch<0x34;ch+=2){
      printf("%c",mix.buffer[ch+1]);
      printf("%c",mix.buffer[ch]);
   }
   printf("\n");


   printf("Input filename : ");
      scanf("%s",file_name);
      strupr(file_name);
      if (!strcmp("Q",file_name) || !strcmp("QUIT",file_name))
	 return;
      printf("Choice Backup-[0]2M,[1]10M,[2]64M,[3]96M,[4]128M,[5]192M,[6]256M Quit : ");
      ch=getche();
      printf("\n");
      switch (ch){
	 case '0' : if (backup_rom_all(8)!=0){		// 8*32K=2Mbit
		       printf("\nbackup error !!!\n");
		       return;
		       } break;
	 case '1' : if (backup_rom_all(40)!=0){		// 40*32K=10Mbit
		       printf("\nbackup error !!!\n");
		       return;
		       } break;
	 case '2' : if (backup_rom_all(256)!=0){        // 256*32K=64Mbit
		       printf("\nbackup error !!!\n");
		       return;
		       } break;
	 case '3' : if (backup_rom_all(384)!=0){	// 384*32K=96Mbit
		       printf("\nbackup error !!!\n");
		       return;
		       } break;
	 case '4' : if (backup_rom_all(512)!=0){	// 512*32K=128Mbit
		       printf("\nbackup error !!!\n");
		       return;
		       } break;
	 case '5' : if (backup_rom_all(768)!=0){	// 768*32K=192Mbit
		       printf("\nbackup error !!!\n");
		       return;
		       } break;
	 case '6' : if (backup_rom_all(1024)!=0){	// 1024*32K=256Mbit
		       printf("\nbackup error !!!\n");
		       return;
		       } break;
      }
//   }
}

void test_sram(void)
{
   if (check_card()!=0){
      printf("\nNo DN64 card present!!!\07\n");
      exit(-1);
   }
   for (i=0;i<0x4000;i++){
      mix.bufferx[i]=i;
   }
   hi_word=0x800;
   if (read_verify_sram(0,0x8000)){		//verify
      printf("Write Pattern to sram \n");
      write_sram();
   }
   else{
      for (hi_word=0x801;hi_word<0x805;hi_word++){
	 if (read_verify_sram(0,0x8000)==0){	// verify sram 32k
	    break;
	 }
      }
      if (hi_word==0x801){
	 printf("sram 32K present\n");
      }
      else
	 if (hi_word==0x804){
	  printf("sram 256K present\n");
	  return;
	 }
	 else {
	    printf("sram check error! (hi_word=%x)\n",hi_word);
	 }
   }
   if (read_verify_sram(0,0x8000))
      printf("no sram present!\07\n");
   else{
      for (hi_word=0x801;hi_word<0x805;hi_word++){
	 if (read_verify_sram(0,0x8000)==0){	// verify sram 32k
	    break;
	 }
      }
      if (hi_word==0x801){
	 printf("sram 32K present\n");
      }
      else
	 if (hi_word==0x804){
	  printf("sram 256K present\n");
	 }
	 else
	    printf("sram check error! (hi_word=%x)\n",hi_word);
   }
}


/*************************************************
*                  MAIN ENTRY                    *
*************************************************/
void main(void)
{
   unsigned char ch;		// single key function
//   for (hi_word=0;hi_word<0x1100;hi_word+=0x80){
//   no_select();			// don't select any device
   printf("--- Parallel EPP port to N64 card program version 1.0 ---\n");
   printf("support 32K sram read/write/verify/save/load/test\n");
   printf("support 512 bytes eeprom read/write/verify/save/load\n");
   printf("support game cart backup\n");
   if (check_card()!=0){
      printf("\nNo DN64 card present!!!\07\n");
      exit(-1);
   }
   while (ch!='q' && ch!='Q'){
      init_port();
      printf("Choice Read/Write/Verify/Save/Load/Dump/Backup/Test_DS1/{1/2/3[r/w/v eeprom]}/Quit(R/W/V/S/L/D/B/T/1/2/3/Q) : ");
      ch=getche();
      printf("\n");
      hi_word=0x0800;
      switch (ch){
	 case 'l' : load_32k_file(); break;
	 case 'L' : load_32k_file(); break;
	 case 's' : save_32k_file(); break;
	 case 'S' : save_32k_file(); break;
	 case 'd' : disp_buffer(512); break;
	 case 'D' : disp_buffer(512); break;
	 case 'r' : read_verify_sram(1,0x8000); break;
	 case 'R' : read_verify_sram(1,0x8000); break;
	 case 'w' : write_sram(); break;
	 case 'W' : write_sram(); break;
	 case 'v' : if (read_verify_sram(0,0x8000)) printf("Verify error!\07\n"); break;
	 case 'V' : if (read_verify_sram(0,0x8000)) printf("Verify error!\07\n"); break;
	 case 'b' : backup(); break;
	 case 'B' : backup(); break;
	 case 't' : test_sram(); break;
	 case 'T' : test_sram(); break;
	 case '1' : read_eeprom_all(); break;
	 case '2' : write_eeprom_all(); break;
	 case '3' : verify_eeprom_all(); break;
      }
      end_port();
   }


   exit(0);







   printf("hi_word=%x ",hi_word);
//   printf("read_data= ");
   for (i=0;i<0x40;i++){
//      printf("%x ",read_data());
	mix.bufferx[i]=inport(data);
   }
   printf("\n");
//   no_select();			// don't select any device
   end_port();
   disp_buffer(0x80);
/*
   set_adr(hi_word,0x0000);
   write_data(0x63ea);		// write data to sram
   no_select();			// don't select any device
   set_adr(hi_word,0x0002);
   write_data(0xd421);		// write data to sram
   no_select();			// don't select any device
   set_adr(hi_word,0x0004);
   write_data(0xfba2);		// write data to sram
   no_select();			// don't select any device


   set_adr(hi_word,0x0000);
   printf("hi_word=%x ",hi_word);
   printf("read_data= ");
   for (i=0;i<8;i++){
      printf("%x ",read_data());
   }
   printf("\n");
   no_select();			// don't select any device
//   }

*/

   exit(0);
}

